#ifndef INCLUDED_CURVES_H
#define INCLUDED_CURVES_H

#pragma warning(disable : 4786)

#include "Curve.h"

#include <iostream>

#include <vector>

#include "Singleton.h"

class Curves : public Singleton<Curves>
{
	public:

		Curves(void);

		Curves(std::istream & input_stream);

		void toStream(std::ostream & output_stream);

		void fromStream(std::istream & input_stream);

		void addCurve(const std::string & curve_name, const double & minimum_value, const double & maximum_value, const double & start_value);

		Curve * getCurve(const std::string & curve_name);

		typedef std::vector<Curve> CurveList;

		typedef CurveList::const_iterator const_iterator;

		const_iterator begin() const;
		const_iterator end() const;

		typedef CurveList::iterator iterator;

		iterator begin();
		iterator end();

		unsigned int getNumberOfCurves(void) const;

		void clearCurves(void);

		void setWrapControlPointsFlags(const bool & wrapping);

		void getClosestControlPointOnShownCurve(const Point & point, double * closest_control_point_distance, Curve * * closest_control_point_curve, unsigned int * closest_control_point_index);

		void getClosestShownCurve(const Point & point, double * closest_curve_distance, Curve * * closest_curve);

		void startZoomSelection(const double & left, const double & bottom);

		void finishZoomSelection(const double & right, const double & top);

		void pushZoom(void);

		void popZoom(void);

		bool canZoomOut(void) const;

		void changeAnimationLength(const double & animation_length);

		double getTime(const double & normalized_x) const;

		void drawAllCurves(void) const;

	private:

		CurveList m_curves;
};

std::ostream & operator<<(std::ostream & output_stream, Curves & curves);

std::istream & operator>>(std::istream & input_stream, Curves & curves);

#endif